#ifndef CLOTH_HH
#define CLOTH_HH

#include "bbox.hh"
#include "vector.hh"
#include "vertex.hh"
#include <string>

class Cloth {
private:
  const int xSize, ySize;
  Vertex* points;
  BBox* boundingTree;

  const double dist, d2, strength, s2, curve, weight, elongation, damping;
  double maxDist, maxD2;
  const bool inters;
public:
  Cloth (std::istream& in, std::string version, int ixSize, int iySize, double idist,
         double id2, double istrength, double is2, double icurve, double iweight,
         double hNoise, double vNoise, double ielongation, double idamping,
         bool iinters);
  Cloth (int ixSize, int iySize, double idist, double id2,
         double istrength, double is2, double icurve, double iweight, 
         double hNoise, double vNoise, double ielongation, double idamping,
         bool iinters);
  Cloth (int ixSize, int iySize, double idist, double id2,
         double istrength, double is2, double icurve, double iweight, 
         double hNoise, double vNoise, double ielongation, double idamping,
         bool iinters, const Vector& orig, const Vector& dir1, const Vector& dir2);
  ~Cloth();

  void translate (const Vector& v);
  void rotate (const Vector& v);

  void simulate (int steps);

  void output_cyl (std::ostream& out) const;
  void output (std::ostream& out) const;
  void output2 (std::ostream& out) const;

  void write (std::ostream& out) const;

  void attach (double u, double v, const Vector& where, const Vector& speed = O);

private:
  void initLinks();

  void computeForces();
  void move();
  void intersect();

  void computeNormals() const;

  void createBoundingTree();

  inline const Vertex& point (int x, int y) const { return points[y*xSize+x]; }
  inline Vertex& point (int x, int y) { return points[y*xSize+x]; }
  inline Vector& normal (int x, int y) const { return points[y*xSize+x].normal; }

  friend class Vertex;
};

#else

class Cloth;

#endif
