#include "common.hh"
#include "cylinder.hh"
#include <cmath>

Cylinder::Cylinder (const Vector& begin, const Vector& end, double rad, double ifriction):
  Object (ifriction), start(begin), dir((end-begin).normalize()), r(rad), r2(rad*rad), 
  l((end-begin).length()) {}

const bool Cylinder::inside (Vector& point, const Vector& old) const {
  Vector diff1 = point-start;
  double d1 = diff1.dot (dir);
  if ((d1 > 0) && (d1 < l)) {
    Vector tmp1 = diff1 - d1*dir;
    if (tmp1.l2() < r*r) {
      Vector diff2 = old-start;
      double d_2 = diff2.dot (dir);
      Vector tmp2 = diff2 - d_2*dir;
      double lambda;
      if (d_2 > 0) {            // do we need to stop on a cap or on the side?
        if(d_2 < l) {           // side
          //lambda = (r-tmp2.length())/(EPSILON+tmp1.length()-tmp2.length());
          Vector op = tmp1-tmp2;
          double dot = tmp2.dot (op);
          lambda = -0.99*(dot + sqrt (dot*dot - op.l2()*(tmp2.l2()-r2)))/op.l2();
        } else {                // top cap
          lambda = (l-d_2)/(-EPSILON+d1-d_2);
        }
      } else {                  // bottom cap
        lambda = -d_2/(-EPSILON+d1-d_2);
      }
      point = old+lambda*(point-old);
      return true;
    }
  }
  return false;
}

const void Cylinder::adjust (Vector& force, const Vector& point) const {
  Vector diff = point - start;
  double d1 = diff.dot (dir);
  if (d1 <= -EPSILON) return;
  if (d1 >= l+EPSILON) return;
  Vector u = (diff - d1*dir);
  if (u.l2() >= r*r+EPSILON) return;
  if (d1 >= l) {                // on the top plane
    computeFriction (force, dir);
  } else if (d1 <= 0) {         // on the bottom plane
    computeFriction (force, -dir);
  } else {                      // neither, therefore on the side
    computeFriction (force, u.normalize());
  }
}

const bool Cylinder::isInside (const Vector& point) const {
  Vector diff = point - start;
  double d1 = diff.dot (dir);
  if ((d1 <= 0)||(d1 >= l)) return false;
  return ((diff - d1*dir).l2() < r*r);
}
