%{
#include <cmath>
#include <cstring>
#include "vector.hh"
#include "parser.hh"

#include <stack>

  using namespace std;

static int currline = 1;

stack<int> state_stack;

#define PUSH_STATE(state) state_stack.push(YY_START); BEGIN(state);
#define POP_STATE BEGIN(state_stack.top()); state_stack.pop();

 void yyerror (char* s);

%}

%x COMMENT

%%

  state_stack.push (INITIAL);

[0-9]+"."?[0-9]*([Ee][+-]?[0-9]+)? { yylval.dbl = atof (yytext); return NUMBER; }
\"[^\"\n\r]*\" { yylval.str = new char[(strlen (yytext)-1)]; strncpy (yylval.str, yytext+1, (strlen (yytext)-2)*sizeof(char)); return STRING; }

"," { return COMMA; }
"{" { return LCURL; }
"}" { return RCURL; }
"<" { return LBRACKET; }
">" { return RBRACKET; }
"(" { return LPAREN; }
")" { return RPAREN; }

global_settings { return GLOBAL_SETTINGS; }
step            { return STEP;            }
output          { return OUTPUT;          }
mesh            { return MESH;            }
mesh2           { return MESH2;           }
cylinders       { return CYLINDERS;       }
wind            { return WIND;            }
viscosity       { return VISCOSITY;       }
gravity         { return GRAVITY;         }

cloth         { return CLOTH;         }
x_size        { return XSIZE;         }
y_size        { return YSIZE;         }
dist          { return DIST;          }
strength      { return STRENGTH;      }
shear         { return SHEAR;         }
stiffness     { return STIFF;         }
weight        { return WEIGHT;        }
hnoise        { return HNOISE;        }
vnoise        { return VNOISE;        }
elongation    { return ELONGATION;    }
damping       { return DAMPING;       }
read_file     { return READ_FILE;     }
position      { return POSITION;      }
intersections { return INTERSECTIONS; }

translate { return TRANSLATE; }
rotate    { return ROTATE;    }
scale     { return SCALE;     }
attach    { return ATTACH;    }

ground       { return GROUND;   }
box          { return BOX;      }
cylinder     { return CYLINDER; }
sphere       { return SPHERE;   }
height_field { return HF;       }

tga { return TGA; }

on    { return ON;  }
true  { return ON;  }
off   { return OFF; }
false { return OFF; }

x   { return X_VECTOR; }
y   { return Y_VECTOR; }
z   { return Z_VECTOR; }
"+" { return PLUS;     }
"-" { return MINUS;    }
"*" { return TIMES;    }
"/" { return OVER;     }

[ \t]+          /* eat up whitespace */

<*>(\n|\n\r|\r\n|\r) currline++;

([[:alnum:]_]+|.) { cerr << endl << "Unrecognized input on line " << currline 
                         << ": '" << yytext << "'!" << endl; exit (1); }

"//"[^\r\n]* /* eat up one-line comments */

<*>"/*"                 { PUSH_STATE (COMMENT); }
<COMMENT>[^*\n\r]*      /* eat up anything that's not a "*" */
<COMMENT>"*"+[^*/\n\r]* /* eat up stars not followed by a "/" */
<COMMENT>"*"+"/"        { POP_STATE; }
<COMMENT><<EOF>>        { yyerror ("Unexpected end of file: unclosed comment!"); }

%%

extern yyFlexLexer lexer;

int yylex() {
  return lexer.yylex();
}

int yywrap() {
  return 1;
}

void yyerror (char* s) {
  cerr << "Error on line " << currline << ": " << s << endl;
  exit (1);
}
