#include "box.hh"
#include "cloth.hh"
#include "cmdline.h"
#include "common.hh"
#include "cylinder.hh"
#include "ground.hh"
#include "object.hh"
#include "vector.hh"
#include "vertex.hh"
#include <iostream>
#include <fstream>
#include <cstdlib>

#include "FlexLexer.h"

using namespace std;

list<Object*> world;
Cloth* cloth = (Cloth*)-1;
double step = 0.2;
Vector wind = O;
Vector gravity = -0.02*Z;
double visc = 0.05;
int nIntersectChecks = 0;
int nIntersectFound = 0;
OutputType output_type = mesh2;
yyFlexLexer lexer (&cin);

int yyparse();

int main (int argc, char **argv) {
  gengetopt_args_info args_info;
  if (cmdline_parser (argc, argv, &args_info) != 0)
    exit(1) ;

  if (args_info.in_file_given)
    lexer.switch_streams (new ifstream (args_info.in_file_arg), NULL);

  yyparse();
  int steps = 1000;
  if (args_info.steps_given)
    steps = args_info.steps_arg;
  cloth->simulate (steps);

  ostream* out = &cout;
  if (args_info.out_file_given)
    out = new ofstream (args_info.out_file_arg);
  out->setf (ios::showpoint);
  out->setf (ios::showpos);
  out->setf (ios::scientific, ios::floatfield);
  switch (output_type) {
  case mesh: cloth->output (*out); break;
  case mesh2: cloth->output2 (*out); break;
  case cylinders: cloth->output_cyl (*out); break;
  }
  if (args_info.out_file_given)
    delete out;
  if (args_info.raw_output_given) {
    out = new ofstream (args_info.raw_output_arg);
    cloth->write (*out);
    delete out;
  }
  delete cloth; cloth = (Cloth*)-1;
  cerr << "Number of self-intersection checks: " << nIntersectChecks << endl
       << "Number of self-intersections found: " << nIntersectFound << endl;
  return 0;
}
