#include "common.hh"
#include "sphere.hh"
#include "utils.hh"
#include <cmath>

Sphere::Sphere (const Vector& icenter, double rad, double ifriction):
  Object (ifriction), center (icenter), r (rad), r2(rad*rad) {}

const bool Sphere::inside (Vector& point, const Vector& old) const {
  Vector diff1 = point-center;
  if (diff1.l2() > r2) return false;
  Vector diff2 = old-center;
  if (diff2.l2() < r2+EPSILON) {
    point = old;
    return true;
  }
  // This isn't the true intersection, but it's a point *outside* the sphere, closer
  // than the original and it should compute faster than a true intersection
  //point = old - ((r-diff2.length())/(diff2.length()-diff1.length()))*(point-old);

  Vector op = point - old;
  double dot = diff2.dot (op);
  point = old - (0.99*(dot + sqrt (dot*dot - op.l2()*(diff2.l2()-r2)))/op.l2())*op;
  /*if ((point-center).l2() <= r2) {
    cerr << endl
         << "Old:    " << old << ", " << diff2.length() << endl
         << "Point:  " << (old+op) << ", " << diff1.length() << endl
         << "New:    " << point << endl
         << "Center: " << center << endl
         << "R:      " << r << endl
         << "R^2:    " << r2 << endl
         << "T:      " << (dot + sqrt (dot*dot - op.l2()*(diff1.l2()-r2)))/op.l2() << endl;
    exit (1);
  }*/
  //if (isInside (point)) segFault();

  return true;
}

const void Sphere::adjust (Vector& force, const Vector& point) const {
  Vector diff = point-center;
  if (diff.l2() >= r2+EPSILON) return;
  computeFriction (force, diff.normalize());
}

const bool Sphere::isInside (const Vector& point) const {
  return ((point-center).l2() < r2);
}
