#include "common.hh"
#include "utils.hh"
#include <cmath>

int solveLin (const double poly[], double result[]) {
  if (poly[1] == 0) return 0;
  result[0] = -poly[0]/poly[1];
  return 1;
}

int solveQuadric (const double poly[], double result[]) {
  if (poly[2] == 0) return solveLin (poly, result);
  double delta = poly[1]*poly[1] - 4*poly[2]*poly[0];
  if (delta < 0) return 0;
  delta = sqrt (delta);
  result[0] = (-poly[1]-delta)/(2*poly[2]);
  result[1] = (-poly[1]+delta)/(2*poly[2]);
  return 2;
}

int solveCubic (const double poly[], double result[]) {
  if (fabs (poly[3]) < EPSILON) return solveQuadric (poly, result);
  double a0 = poly[0]/poly[3];
  double a1 = poly[1]/poly[3];
  double a2 = poly[2]/poly[3];
  double A2 = a2*a2;
  double Q = (A2 - 3*a1)/9;
  double R = (a2 * (A2 - 4.5*a1) + 13.5*a0) / 27;
  double Q3 = Q*Q*Q;
  double R2 = R*R;
  double d = Q3 - R2;
  double an = a2/3;

  if (d >= 0) {
    d = R / sqrt (Q3);
    double theta = acos(d)/3;
    double sQ = -2*sqrt(Q);

    result[0] = sQ*cos (theta) - an;
    double tmp = sQ*cos (theta + TWO_PI_3) - an;
    if (tmp >= result[0]) {
      result[1] = tmp;
    } else {
      result[1] = result[0];
      result[0] = tmp;
    }
    tmp = sQ*cos (theta - TWO_PI_3) - an;
    if (tmp >= result[1]) {
      result[2] = tmp;
    } else {
      result[2] = result[1];
      if (tmp >= result[0]) {
        result[1] = tmp;
      } else {
        result[1] = result[0];
        result[0] = tmp;
      }
    }
    return 3;
  } else {
    double sQ = pow (sqrt (R2-Q3) + fabs (R), 1/3.0);
    if (R < 0)
      result[0] = (sQ + Q/sQ) - an;
    else
      result[0] = -(sQ + Q/sQ) - an;
    return 1;
  }
}

void segFault() {
  int* bogus = (int*)(-1);
  delete bogus;
}
