#include "common.hh"
#include "utils.hh"
#include "vector.hh"
#include <cstring>
#include <cmath>

using namespace std;

const Vector O (0, 0, 0);
const Vector X (1, 0, 0);
const Vector Y (0, 1, 0);
const Vector Z (0, 0, 1);

static class VectorInit {
public:
  VectorInit() {
    O.length(); X.length(); Y.length(); Z.length();
  }
} initializer;

Vector::Vector (double x, double y, double z): coords (), l_(-1.0), l2_(-1) {
  coords[0] = x;
  coords[1] = y;
  coords[2] = z;
}

Vector::Vector (const Vector& v): coords(), l_(), l2_() {
  memcpy (coords, v.coords, sizeof (coords));
  l_ = v.l_;
  l2_ = v.l2_;
}

const Vector Vector::operator+ (const Vector& v) const {
  return Vector (coords[0]+v.coords[0], coords[1]+v.coords[1], coords[2]+v.coords[2]);
}
const Vector Vector::operator+() const {
  return *this;
}
const Vector Vector::operator- (const Vector& v) const {
  return Vector (coords[0]-v.coords[0], coords[1]-v.coords[1], coords[2]-v.coords[2]);
}
const Vector Vector::operator-() const {
  return Vector (-coords[0], -coords[1], -coords[2]);
}

Vector& Vector::operator+= (const Vector& v) {
  coords[0] += v.coords[0];
  coords[1] += v.coords[1];
  coords[2] += v.coords[2];
  l_ = l2_ = -1;
  return *this;
}
Vector& Vector::operator-= (const Vector& v) {
  coords[0] -= v.coords[0];
  coords[1] -= v.coords[1];
  coords[2] -= v.coords[2];
  l_ = l2_ = -1;
  return *this;
}
Vector& Vector::operator*= (const double l) {
  coords[0] *= l;
  coords[1] *= l;
  coords[2] *= l;
  l_ = l2_ = -1;
  return *this;
}

const Vector operator* (const double l, const Vector& v) {
  return Vector (l*v.coords[0], l*v.coords[1], l*v.coords[2]);
}

ostream& operator<< (ostream& o, const Vector& v) {
  o << "<" << v.x() << ", " << v.z() << ", " << v.y() << ">";
  return o;
}

const bool Vector::operator== (const Vector& v) const {
  return ((*this)-v).length() < EPSILON;
}

const double Vector::length() const {
  if (l_ < 0)
    l_ = (l2()>EPSILON)?sqrt (l2_):0;
  return l_;
}

const double Vector::l2() const {
  if (l2_ < 0)
    l2_ = coords[0]*coords[0] + coords[1]*coords[1] + coords[2]*coords[2];
  if (!isfinite(l2_)) {
    cerr << endl << l2_ << " is not finite!" << endl
         << (*this) << endl;
    segFault();
    exit(1);
  }
  return l2_;
}

const Vector Vector::normalize() const {
  if (length() < EPSILON) {
    cerr << endl << "Attempting to normalize zero length vector!" << endl;
    exit (1);
  }
  double l = 1/length();
  return l*(*this);
}
const double Vector::dot (const Vector& v) const {
  return coords[0]*v.coords[0] + coords[1]*v.coords[1] + coords[2]*v.coords[2];
}
const Vector Vector::cross (const Vector& v) const {
  return Vector (coords[1]*v.coords[2]-coords[2]*v.coords[1],
                 coords[2]*v.coords[0]-coords[0]*v.coords[2],
                 coords[0]*v.coords[1]-coords[1]*v.coords[0]);
}
