#ifndef VERTEX_HH
#define VERTEX_HH

#include "common.hh"
#include "vector.hh"
#include <list>

class Cloth;
class Vertex {
private:
  Vector location_, speed, force, normal, newloc, adjusted, oldLoc, oldForce, oldSpeed;
  int nadj;

  std::list<Vertex*> neighbors, corners;
  std::list<std::pair<Vertex*, Vertex*> > curvings;

  Cloth* owner;

  bool fixed, attached;

public:
  Vertex();
  Vertex (const Vector&, Cloth* iowner);

  void addNeighbor (Vertex*);
  void addCorner (Vertex*);
  void addCurving (Vertex*, Vertex*);
  void attach (const Vector&, const Vector& whatSpeed = O);

  void computeForce();
  void adjust();
  void move();
  void doMove();
  void reset() { force = O; }

  void adjustLength();
  void endAdjust();
  void intersect (Vertex&, Vertex&, Vertex&);

  const bool moving() const { return (force.length() >= EPSILON); }

  inline const Vector& location() const { return location_; }
  inline const Vector& newl() const { return newloc; }

  friend class Cloth;
};

#include "cloth.hh"

#else

class Vertex;

#endif
